<style>
	body {font-family:Arial;}
	table th, table td {border: 1px solid #ff960f;padding:10px;}
	table thead {background:#f1d7ce;}
	table tbody tr:nth-child(2n) {background:#f1d7ce4d;}
</style>
<?php
/* Funds Capture */
require 'vendor/autoload.php';
require 'config.php';

//Update Execution Time Limit
set_time_limit(600); 

function fetchAuthorizedOrders($shopify, $storeURL, $orderCursor = null, $lastId = null) {
/* This query fetches paginates through all authorized and capturable orders.
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, totalPriceSet.shopMoney.amount
*/
	
//If Cursor, Add in pagination
if (!is_null($orderCursor) && !empty($orderCursor)) {
	$orderCursorText = ', after: "'.$orderCursor.'"';
} else {
	$orderCursorText = '';
}
		
$gQLRequest =
<<<Query
query {
  orders(first: 20, query: "(fulfillment_status:shipped) AND (financial_status:authorized)" $orderCursorText) {
	pageInfo {
		hasNextPage
		hasPreviousPage
	}
	edges {
	  cursor
	  node {
		id
		transactions {
		  id
		}
		totalPriceSet {
		  shopMoney {
			amount
		  }
		}
		totalRefundedShippingSet {
		  shopMoney {
		    amount
		  }
		}
		totalShippingPriceSet {
		  shopMoney {
		    amount
		  }
		}
		taxLines {
          ratePercentage
		}
        lineItems (first:10) {
			pageInfo {
              hasNextPage
              hasPreviousPage
            }
			edges {
				cursor
				node {
					id
					name
					refundableQuantity
					discountedUnitPriceSet {
						shopMoney {
							amount
						}
					}  
					taxLines {
					  rate
					}
				}
			}
		}
	  }
	}
  }
}
Query;
	$data = $shopify->GraphQL->post($gQLRequest);
	
	

	if ( count($data["data"]["orders"]["edges"]) > 0) {

		//Checks for paginatinated results
		$hasPreviousPage = $data["data"]["orders"]["pageInfo"]["hasPreviousPage"];
		$hasNextPage 	 = $data["data"]["orders"]["pageInfo"]["hasNextPage"];
		$orderCursor 	 = $data["data"]["orders"]["edges"][0]["cursor"];
		
		if ($hasPreviousPage == false || empty($hasPreviousPage)) {
			echo 
			"<table>
			<thead>
				<tr>
					<th>Order ID</th>
					<th>Transaction ID</th>
					<th>Capturable Total</th>
					<th>Updated Total (Including Shipping &amp; Tax)</th>
				</tr>
			</thead>
			<tbody>";
		}
		
		foreach($data["data"]["orders"]["edges"] as $order) {
			/*
			* 	For each order, capture payment
			*/
			$lineItemHasNextPage = $order["node"]["lineItems"]["pageInfo"]["hasNextPage"];
			$transaction = end($order["node"]["transactions"]);
			$orderId  = $order["node"]["id"];
			$orderArr = explode("/", $orderId);
			$orderNum = end($orderArr);

			// Tally capturable total.
			$updatedTotal = 0;
			
			foreach($order["node"]["lineItems"]["edges"] as $lineItem) {
				// Calculate line item subtotal and rounded taxes
				$unitPrice 	 = floatval($lineItem["node"]["discountedUnitPriceSet"]["shopMoney"]["amount"]);
				$orderQty 	 = floatval($lineItem["node"]["refundableQuantity"]);
				$updatedTotal += $unitPrice * $orderQty;
				
				// Round and add taxes to total 
				/*  Developer Note:
					This is On^3, not ideal but it's hard to be avoided and still serve our purposes.				
					//In this case, this gets us a rounded tax slightly above the shopify calculated tax total, but will "complete the capture", and refund them a few cents
					// The version of this without the foreach loop calculates below shopify's total, and caused the order to be partially paid needing the customer to pay again and for us to recapture/reauthorize that amount
					// Usually I would just use an array of 1-2 taxes for state and local, but we don't know how many taxes will be there per order or for international users
					// EX: People in the Europe would get VAT taxes, or other taxes added in that will needed to be individually taxed per lineitem and tallied.
					// If we find a better solution, we can refactor this below.
				*/
				foreach($order["node"]["taxLines"] as $taxRate) {
					$updatedTotal +=  round(($unitPrice * (floatval($taxRate["ratePercentage"]) / 100)), 2) * $orderQty;
				}
				$lineItemCursor = $lineItem["cursor"];
			}
			
			// If more lineitems available paginate through them
			if (isset($lineItemCursor) && !empty($lineItemHasNextPage) && $lineItemHasNextPage !== false) {
				$updatedTotal = getPaginatedTotal($shopify, $storeURL, $orderId, $lineItemCursor, null, $updatedTotal);
			}
			
			// Add In Shipping Cost to the order
			$updatedTotal += (floatval($order["node"]["totalShippingPriceSet"]["shopMoney"]["amount"]) - floatval($order["node"]["totalRefundedShippingSet"]["shopMoney"]["amount"]));	
			
			if ($lastId !== $orderId) {
				echo 
				"<tr>
					<td><a href=\"https://". $storeURL ."/admin/orders/".$orderNum."\">".$orderNum."</a></td>
					<td>".$transaction["id"]."</td>
					<td>$".number_format($order["node"]["totalPriceSet"]["shopMoney"]["amount"], 2)."</td>
					<td>$".number_format($updatedTotal, 2)."</td>
				</tr>";

				// Capture Payment
				captureFunds($shopify, $orderId, $transaction["id"], $updatedTotal);
			}
		}
		
		if ($hasNextPage == false || empty($hasNextPage)) {
			echo "</tbody></table>";
			echo "<h4>&#9989; All Eligible Orders Payments Have been Captured!</h4><a href=\"./fundsCapture.php\">Run Again?</a>";
		} else {
			//Process Next Set Of Orders
			fetchAuthorizedOrders($shopify, $storeURL, $orderCursor, $orderId);
		}
	}
	else {
		echo "<br/><br/><em>&#x274C; There are no Eligible Orders To Capture.</em><br/><a href=\"./fundsCapture.php\">Check Again?</a>";
	}
}
	
	

function getPaginatedTotal($shopify, $storeURL, $orderId = null, $lineItemCursor = null, $lastLineItemId = null, $updatedTotal = 0) {
/* This query fetches paginates through all remaining line items from a Single Order
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, totalPriceSet.shopMoney.amount
*/
	
//If LineItem Cursor, Add in pagination
$lineItemCursorText = (!is_null($lineItemCursor) && !empty($lineItemCursor)) ? ', after: "'.$lineItemCursor .'"' : '';

$gQLRequest =
<<<Query
query {
	order(id: "$orderId") {
		id        
		taxLines {
          ratePercentage
		}
		lineItems (first:20 $lineItemCursorText) {
			pageInfo {
			  hasNextPage
			  hasPreviousPage
			}
			edges {
				cursor
				node {
					id
					name
					refundableQuantity
					discountedUnitPriceSet {
						shopMoney {
							amount
						}
					}
					taxLines {
					  rate
					}
				}
			}
		}
	}
}
Query;
	$data = $shopify->GraphQL->post($gQLRequest);

	// Used to create order url
	$orderId  = $data["data"]["order"]["id"];

	// Checks for paginatinated LineItem results
	$lineItemHasNextPage = $data["data"]["order"]["lineItems"]["pageInfo"]["hasNextPage"];
				
	// Tally capturable total
	foreach($data["data"]["order"]["lineItems"]["edges"] as $lineItem) {
		// Traverse through every lineItem and not overlap lineitems
		if ($lastLineItemId !== $lineItem["node"]["id"]) {
			// Calculate line item subtotal and rounded taxes
			$unitPrice 	 = floatval($lineItem["node"]["discountedUnitPriceSet"]["shopMoney"]["amount"]);
			$orderQty 	 = floatval($lineItem["node"]["refundableQuantity"]);
			$updatedTotal += $unitPrice * $orderQty;
			
			// Round and add taxes to total
			foreach($data["data"]["order"]["taxLines"] as $taxRate) {
				$updatedTotal +=  round(($unitPrice * (floatval($taxRate["ratePercentage"]) / 100)), 2, PHP_ROUND_HALF_DOWN) * $orderQty;
			}
		
			$lineItemCursor = $lineItem["cursor"];
			$lastLineItemId = $lineItem["node"]["id"];
		}
	}

	// Process Next LineItem Batch from current order & add to refund total
	if (!empty($lineItemHasNextPage) && $lineItemHasNextPage !== false) {
		return floatval(getPaginatedTotal($shopify, $storeURL, $orderId, $lineItemCursor, $lastLineItemId, $updatedTotal));
	} else {	
		return floatval($updatedTotal);
	}
}	
	
	
	
	
	
function captureFunds($shopify, $orderId, $transactionId, $orderTotal) {
/* This query fetches paginates through all authorized and capturable orders.
*  https://shopify.dev/docs/admin-api/rest/reference/orders/order
*  Extracted fields: order.id, transactions.id, totalPriceSet.shopMoney.amount
*/
$input = '{ id:"'.$orderId.'", parentTransactionId:"'.$transactionId.'", amount:"'.$orderTotal.'" }';

$gQLRequest =
<<<Query
mutation {
  orderCapture(input: $input) {
    transaction {
      id
    }
    userErrors {
      field
      message
    }
  }
}
Query;

$data = $shopify->GraphQL->post($gQLRequest);	
//print("<pre>commitOrderUpdate ".print_r($data,true)."</pre>");
}
	
echo "<h2>NatureMade&reg; | Funds Capture</h2> <em>Store: ".$config["ShopUrl"]."</em>";
// Fetch Orders & Capture Authorized Payments	
fetchAuthorizedOrders($shopify, $config["ShopUrl"]);


?>